class StationNetwork:
    def __init__(self, code: str = None, id: int = None, description: int = None, client=None):
        self.id = id
        self.code = code
        self.description = description

        self._client = client

    @property
    def url(self):
        if self.code is not None:
            return "station-networks/" + self.code
        else:
            return "station-networks/id:" + str(self.id)





from cgdb.managers.sitesManager import SitesManager

class StationGroup:
    def __init__(self,  id: int = None, code: str = None, name=None, description: int = None, client=None):
        self.id = id
        self.code = code
        self.name = name
        self.description = description

        self._client = client

    @property
    def url(self):
        if self.code is not None:
            return "station-groups/" + self.code
        else:
            return "station-groups/id:" + str(self.id)

    @property
    def sites(self):
        sites_manager = SitesManager(client=self)
        return sites_manager.sites(self.url)



import json

from cgdb.resources.level import Level
from cgdb.utils.ManagerMix import ManagerMix


class LevelsManager(ManagerMix):
    def __init__(self, client):
        super().__init__(client)

    def levels(self):
        content = self.get("levels")
        time_steps = []

        for raw in content:
            time_steps.append(Level(**raw))

        return time_steps

    def level_by_id(self, id):
        content = self.get("levels/id:" + str(id))

        return Level(**content)

    def level_by_code(self, code: str):
        content = self.get("levels/" + str(code))

        return Level(**content)

    def create_level(self, code, description, level):
        out = {"code": code,
                "level": level,
                "description": description}
        result = self.post("levels", json.dumps(out))
        if result.status_code != 200 and result.status_code != 204:
            raise Exception("upload fail on server side\n Server error:"+result.content.decode("ASCII"))




class ColumnNotFoundException(Exception):
    def __init__(self, column_name, message="Column in data frame not fond"):
        self.column_name = column_name
        self.message = message
        super().__init__(self.message)


from cgdb.resources.timeStep import TimeStep
from cgdb.utils.ManagerMix import ManagerMix


class TimeStepsManager(ManagerMix):
    def __init__(self, client):
        super().__init__(client)


    def time_steps(self):
        content = self.get("time-steps")
        time_steps = []

        for raw in content:
            time_steps.append(TimeStep(**raw))

        return time_steps


    def time_step_by_id(self, id):
        content = self.get("time-steps/id:" + str(id))

        return TimeStep(**content)

    def time_step_by_code(self, code: str):
        content = self.get("time-steps/" + str(code))

        return TimeStep(**content)



from cgdb.utils.ManagerMix import ManagerMix
from cgdb.resources.department import Department


class DepartmentsManager(ManagerMix):
    def __init__(self, client):
        super().__init__(client)

    def departments(self):
        content = self.get("departments")
        departments = []

        for department_raw in content:
            departments.append(Department(**department_raw, client=self._client))

        return departments

    def department(self, id: str):
        content = self.get("departments/" + id)

        return Department(*content, client=self._client)



class Element:
    def __init__(self, mark: str=None, id: int = None, categoryId: int = None, description: str = None, defaultUnit: str = None, defaultColor: str = None, chartType: str = None, client=None) -> None:
        super().__init__()
        self.mark = mark
        self.id = id
        self.categoryId = categoryId
        self.description = description
        self.defaultUnit = defaultUnit
        self.defaultColor = defaultColor
        self.chartType = chartType

        self._client = client

    @property
    def url(self):
        if self.mark is not None:
            return "elements/" + self.mark
        else:
            return "elements/id:" + str(self.id)

    def data_sets(self):
        return self._client._data_sets_manager.data_sets(self.url)

    def flag_lists(self):
        return self._client._flag_lists_manager.flag_lists(self.url + "/flags")

    def add_flag(self, name):
        current_flag_list_names = [flag.code for flag in self.flag_lists()]

        



class Parameter:
    def __init__(self, id:int = None, name:str = None, description:str=None, type:str=None, created:int=None, last_updated:int=None, client=None):
        self.id = id
        self.name = name
        self.description = description
        self.type = type
        self.created = created
        self.last_updated = last_updated

        self.client = client


import json

from cgdb.resources.stationGroup import StationGroup
from cgdb.utils.ManagerMix import ManagerMix


class StationGroupsManager(ManagerMix):
    def __init__(self, client):
        super(StationGroupsManager, self).__init__(client)

    def station_groups(self):
        content = self.get("station-groups")
        sites = []

        for station_group_raw in content:
            sites.append(StationGroup(**station_group_raw, client=self._client))

        return sites

    def station_group(self, mark):
        content = self.get("station-groups/" + mark)

        return StationGroup(**content, client=self._client)

    def create_station_group(self, code, name, description):
        out = {"name": name,
                "code": code,
               "description": description}
        result = self.post("station-groups", json.dumps(out))
        if result.status_code != 200:
            raise Exception("upload fail on server side\n Server error:"+result.content.decode("ASCII"))



class EntityNotFoundException(Exception):
    def __init__(self, code, message="Required entity not found"):
        self.code = code
        self.message = message
        super().__init__(self.message)



import datetime
import json

import numpy as np
import pandas as pd
from pandas.api.types import is_datetime64_any_dtype as is_datetime


from cgdb.exceptions.ColumnNotFound import ColumnNotFoundException
from cgdb.resources import OverwriteStrategy
from cgdb.resources.dataSet import DataSets
from cgdb.utils.ManagerMix import ManagerMix


class DataSetsManager(ManagerMix):
    def __init__(self, client):
        super().__init__(client)

    def data_sets(self, url=None, site=None):
        content = self.get(url + "/data-sets")
        data_sets = []

        for station_raw in content:
            data_sets.append(DataSets(**station_raw, client=self._client, site=site))

        return data_sets

    def data_set_by_code(self, site, element, aggregation, time_step, level = None, elementMarkOverride = None):
        if elementMarkOverride is not None:
            element = elementMarkOverride
        code = element + "_" + aggregation + "_" + str(time_step)
        if level is not None:
            code += "_" + str(level)
        content = self.get(site.url + "/data-sets/" + code)

        return DataSets(**content, client=self._client, site=site)

    def get_data_set_data(self, data_set: DataSets, date_from: datetime.datetime = None, date_to: datetime.datetime = None):
        if (date_from is not None and date_to is None) or (date_from is None and date_to is not None):
            print("one date missing")
            return

        url = f"{data_set.url}/data"

        if date_from is not None and date_to is not None:
            date_from = int(date_from.timestamp())
            date_to = int(date_to.timestamp())

            url_param = f"?date-time-from={date_from}&date-time-to={date_to}&page-size=500"
            url += url_param


        content = self.get(url)
        items_count = content["itemsCount"]
        items_count_loaded = 0
        page = 0
        data=None
        # load data from pages
        while items_count_loaded < items_count:
            if items_count_loaded == 0:
                values = content["values"]
            else:
                values = self.get(url+"&page="+str(items_count_loaded))["values"]

            mapping_flags = data_set._mapping_flags_to_value()
            mapping_params = data_set._mapping_parameters_to_col_name_type()

            items_count_loaded += len(values)
            page += 1
            records = []
            for record in values:
                record_to_data_frame = {"timestamp": record["timestamp"], "value": record["value"]}
                if record["flags"] is not None:
                    for flag in record["flags"]:
                        flag_info = mapping_flags[flag]
                        record_to_data_frame[flag_info[0]] = flag_info[1]
                if record["parameters"] is not None:
                    for param_col, param_value in record["parameters"].items():
                        param_info = mapping_params[param_col]
                        record_to_data_frame[param_info[0]] = param_value

                records.append(record_to_data_frame)

            if data is None and len(records) == 0:
                return pd.DataFrame({"datetime": [], "value":[]})

            data_page = pd.DataFrame(records)
            data_page["timestamp"] = pd.to_datetime(data_page['timestamp'], unit="s")
            # data_page.timestamp = data_page.timestamp.dt.tz_localize('UTC').dt.tz_convert('Europe/Brussels')
            data_page.rename(columns={"timestamp": "datetime", "value": data_set.code}, inplace=True)
            for parameter in data_set.parameters:
                if parameter.name in data_page.columns and parameter.type in ["INTEGER", "DOUBLE"]:
                    data_page[parameter.name] = pd.to_numeric(data_page[parameter.name])

            if data is None:
                data = data_page
            else:
                data = pd.concat([data, data_page])

        return data

    # def make_empty_data_frame(self,data_set):
    #     columns = {"timestamp": pd.Series(dtype="int"),
    #                "value": pd.Series(dtype="float")}
    #
    #     for flags_list in data_set.flags:
    #         columns[flags_list.name] = pd.Series(dtype="str")
    #
    #     for parameter in data_set.parameters:
    #         columns[parameter.name] = pd.Series(dtype="str")
    #
    #     return pd.DataFrame(columns)

    def post_data_set_data2(self, data: pd.DataFrame, data_set: DataSets, overwriteStrategy: str = None):
        print(data)

        if "date_time" not in data.columns:
            raise ColumnNotFoundException("date_time")
        elif "value" not in data.columns:
            raise ColumnNotFoundException("value")

        data = data.dropna(subset=["value"])

        parameters_prefix_columns_data = [column_name for column_name in data.columns if column_name.startswith("parameter-")]
        flags_prefix_columns_data = [column_name for column_name in data.columns if column_name.startswith("flag-")]

        data_set_parameters_names = [parameter.name for parameter in data_set.parameters]
        data_set_flags_names = [flag.name for flag in data_set.flags]

        if not all(column in data_set_parameters_names for column in [column_with_prefix[len("parameter-"):] for column_with_prefix in parameters_prefix_columns_data]):
            raise Exception("parameters columns are not in data set. columns: " + str(parameters_prefix_columns_data) + " in data set: " + str(data_set_parameters_names))
        if not all(column in data_set_flags_names for column in [column_with_prefix[len("flag-"):] for column_with_prefix in flags_prefix_columns_data]):
            raise Exception("flags columns are not in data set. columns: " + str(flags_prefix_columns_data) + " in data set: " + str(data_set_flags_names))

        data = data[["date_time", "value"] + parameters_prefix_columns_data + flags_prefix_columns_data].copy()

        flags_mapping = data_set._mapping_flags_values_to_id()
        parameters_mapping = data_set._mapping_parameters_col_to_id_type()

        for flag_column_data in flags_prefix_columns_data:
            data[flag_column_data] = data[flag_column_data].map(flags_mapping[flag_column_data])

        parameters_id_columns_data = []
        for parameter_column_data in parameters_prefix_columns_data:
            if parameters_mapping[parameter_column_data][1] == "INTEGER":
                try:
                    data[parameter_column_data] = data[parameter_column_data].astype('Int64')
                except TypeError:
                    raise Exception("Paramater column:" + parameter_column_data + " must be integer")
            elif parameters_mapping[parameter_column_data][1] == "DOUBLE" and data.dtypes[parameter_column_data] != np.float64:
                raise Exception("Paramater column:" + parameter_column_data + " must be Double")
            data.rename(columns={parameter_column_data: str(parameters_mapping[parameter_column_data][0])}, inplace=True)
            parameters_id_columns_data.append(str(parameters_mapping[parameter_column_data][0]))

        if is_datetime(data["date_time"]):
            data["date_time"] = (data.date_time.values.astype(np.int64) / 10**9).astype(np.int64)

        out = []
        for index, row in data.iterrows():
            if len(flags_prefix_columns_data) == 0:
                flags = None
            else:
                flags = []
                for col in flags_prefix_columns_data:
                    if not pd.isnull(row[col]):
                        flags.append(int(row[col]))

            if len(parameters_id_columns_data) == 0:
                parameters = None
            else:
                parameters = {}
                for col in parameters_id_columns_data:
                    if not pd.isnull(row[col]):
                        parameters[col] = str(row[col])

            out.append({"timestamp": int(row["date_time"]), "value": float(row["value"]), "parameters": parameters,
                        "flags": flags})

        out = json.dumps(out)


        url = data_set.url + "/data"

        if overwriteStrategy is not None:
            url += "?overwriteStrategy=" + overwriteStrategy

        a = self._client._session.post(data_set.url+"/data", data=out)
        if a.status_code != 200:
            raise Exception("upload fail on server side "+a.text)



    # def post_data_set_data(self, url, data: pd.DataFrame, date_time_column_name:str = "date_time", value_column_name:str = "value", parameter_columns_names:dict = None, flags_columns_names:dict = None, flags=None, parameters=None):
    #     url = f"{url}/data"
    #
    #     if date_time_column_name not in data.columns:
    #         raise ColumnNotFoundException(date_time_column_name)
    #     elif value_column_name not in data.columns:
    #         raise ColumnNotFoundException(value_column_name)
    #     elif not parameters is not None and set(parameter_columns_names.values()).issubset(data.columns):
    #         raise ColumnNotFoundException(','.join([column_name for parameter_name, column_name in parameter_columns_names]))
    #
    #     rename_dict, columns = self._rename_column_dict(date_time_column_name, value_column_name, parameter_columns_names, flags_columns_names)
    #
    #     out = data[columns].copy().dropna()
    #     # content = out.to_json(orient="records")
    #     content = self._prepare_json(out,rename_dict,parameter_columns_names, flags_columns_names, flags=flags, parameters=parameters)
    #
    #     a = self._client._session.post(url, data=content)
    #     if a.status_code != 200:
    #         raise Exception("upload fail")
    #     print(a)
    #
    # def _rename_column_dict(self, date_time_column_name: str, value_column_name: str, parameter_columns_names: dict, flags_columns_names: dict):
    #     out_dict = {date_time_column_name: "timestamp",
    #            value_column_name: "value"}
    #     out_list = [date_time_column_name, value_column_name]
    #
    #     if parameter_columns_names is not None:
    #         for parameter, column_name in parameter_columns_names.items():
    #             out_dict[column_name] = "parameter-" + parameter
    #             out_list.append(column_name)
    #
    #     if flags_columns_names is not None:
    #         for flag, column_name in flags_columns_names.items():
    #             out_dict[column_name] = "flag-" + flag
    #             out_list.append(column_name)
    #
    #     return out_dict, out_list
    #
    # def _prepare_json(self, data, rename_param, parameters_maping, flags_maping, flags, parameters):
    #     data = data.rename(columns=rename_param)
    #     data["timestamp"] = (data.timestamp.values.astype(np.int64) / 10**9) - 3600
    #
    #     parameters_actual = {}
    #     flags_actual = {}
    #     flags_value_maping = {}
    #
    #     if parameters_maping is not None:
    #         for parameter_name in parameters_maping:
    #             for parameters_all_one in parameters:
    #                 if parameters_all_one.name == parameter_name:
    #                     parameters_actual["parameter-"+parameter_name] = parameters_all_one.id
    #
    #     if parameters_maping is not None:
    #         for flag_list_name in flags_maping:
    #             for flags_all_one in flags:
    #                 if flags_all_one.name == flag_list_name:
    #                     flags_actual["flag-"+flag_list_name] = flags_all_one.id
    #                     flags_value_maping["flag-"+flag_list_name] = {}
    #                     for flag in flags_all_one.flags:
    #                         flags_value_maping["flag-"+flag_list_name][flag.code] = flag.id
    #
    #         data.replace(flags_value_maping, inplace=True)
    #     out = []
    #
    #     for index, row in data.iterrows():
    #         if len(parameters_actual.keys()) == 0:
    #             parameters = None
    #         else:
    #             parameters = {}
    #             for param in parameters_actual:
    #                 parameters[str(parameters_actual[param])] = str(int(row[param]))
    #
    #         if len(flags_actual.keys()) == 0:
    #             flags = None
    #         else:
    #             flags = []
    #             for flag in flags_actual:
    #                 flags.append(int(row[flag]))
    #
    #         out.append({"timestamp": int(row["timestamp"]), "value": float(row["value"]), "parameters": parameters, "flags": flags})
    #
    #     return json.dumps(out)

    def create_data_set(self, station, element_code, aggregation, time_step_code, level_code, data_set_type="TECHNICAL", elementMarkOverride = None, description = None):
        element_context = self._client._element_contexts_manager.element_context_by_element(element_code, aggregation, time_step_code, level_code)

        url = station.url + "/data-sets"

        content = {"contextId": element_context.id,
                   "elementMarkOverride": elementMarkOverride,
                   "dataSetType": data_set_type,
                   "description": description}

        self.post(url, json.dumps(content))

    def delete_data_set(self, data_set: DataSets):
        self.delete(data_set.url)



import json

from cgdb.resources.elementContext import ElementContext
from cgdb.utils.ManagerMix import ManagerMix


class ElementContextsManager(ManagerMix):
    def __init__(self, client):
        super().__init__(client)

        self.elementContextsCached = None

    def element_contexts(self):
        content = self.get("element-contexts")
        element_contexts = []

        for element_context_raw in content:
            element_contexts.append(ElementContext(**element_context_raw, client=self._client))

        return element_contexts

    def element_context(self, id):
        content = self.get("element-contexts/id:" + str(id))

        return ElementContext(**content, client=self._client)

    def element_context_by_element(self, element_mark: str, aggregation: str, timestamp_code: str, level_code:str = None):
        if self.elementContextsCached is None:
            self.elementContextsCached = self.element_contexts()

        for elementContext in self.elementContextsCached:
            code = str(element_mark) + "_" + aggregation + "_" + str(timestamp_code)
            if level_code is not None:
                code += "_" + str(level_code)
            # if elementContext.aggregation == aggregation and elementContext.element.mark == element_mark and elementContext.time_step.code == timestamp_code and (level_code is None or (level_code is not None and elementContext.levelId is not None and elementContext.level.code == level_code)):
            #     return elementContext
            if elementContext.code == code:
                return elementContext

        return None
    
    def create_element_context(self, element, aggregation, timeStep, level=None):
        content = {"elementId": element.id,
                   "timeStepId": timeStep.id,
                   "aggregation": aggregation}

        if level is not None:
            content["levelId"] = level.id

        self.post('element-contexts', json.dumps(content))
        self.elementContextsCached = self.element_contexts()



class ElementContext:
    def __init__(self, id: int, elementId: int = None, timeStepId: int = None, code: str = None, aggregation: str = None, unit: str = None, levelId: int = None, client=None):
        self.unit = unit
        self.aggregation = aggregation
        self.levelId = levelId
        self.timeStepId = timeStepId
        self.elementId = elementId
        self.code = code
        self.id = id

        self._client = client

        self.__load_element()
        self.__load_time_step()
        if self.code.count("_") == 3:
            self.__load_level()
        else:
            self.level = None


    @property
    def url(self):
        return "/element-contexts/id:" + str(self.id)

    def __load_element(self):
        self.element = self._client._elements_manager.element("id:"+str(self.elementId))

    def __load_time_step(self):
        self.time_step = self._client._time_steps_manager.time_step_by_id(self.timeStepId)

    def __load_level(self):
        if self.levelId is not None:
            self.level = self._client._levels_manager.level_by_id(self.levelId)


from requests import Response, Session


class CGDBAPISession(Session):
    def __init__(self, host, api_key, *args, **kwargs):
        self.host = host
        self.api_key = api_key
        super(CGDBAPISession, self).__init__(*args, **kwargs)

        self.headers.update({
            'Accept-Charset': 'utf-8',
            'Content-Type': 'application/json',
            'User-Agent': 'cgdb-python-client/{}'.format(0.1)
        })

   # def request(self, method, url: str | bytes | Text, *args, **kwargs) -> Response:
    def request(self, method, url, *args, **kwargs) -> Response:

        if '?' in url:
            url += "&"
        else:
            url += "?"
        url += "token={0}".format(self.api_key)

        return super().request(method, self.host + url, verify=False, *args, **kwargs)


from cgdb.resources.flag import Flag
from cgdb.resources.flagsList import FlagList
from cgdb.utils.ManagerMix import ManagerMix


class FlagListManager(ManagerMix):
    def __init__(self, client):
        super().__init__(client)

    def flag_lists(self, path):
        content = self.get(path)
        flags_lists = []

        for flags_list_raw in content:
            flags_list = FlagList(flags_list_raw["id"],
                                        flags_list_raw["name"],
                                        flags_list_raw["description"],
                                        flags = [],
                                        client=self._client)
            for flag_raw in flags_list_raw["flags"]:
                flags_list.flags.append(Flag(**flag_raw))

            flags_lists.append(flags_list)

        return flags_lists



from cgdb.resources.aggregation import Aggregation


class AggregationsManager:
    def aggregations(self):
        return [Aggregation("VAL"), Aggregation("AVG"), Aggregation("SUM"), Aggregation("MIN"), Aggregation("MAX"),
                Aggregation("TM"), Aggregation("TN"), Aggregation("TE"), Aggregation("MED")]



from typing import List

from cgdb.managers import AggregationsManager
from cgdb.managers import ElementsManager
from cgdb.managers import FlagListManager
from cgdb.managers import SitesManager
from cgdb.managers import DataSetsManager
from cgdb.managers import StationNetworksManager
from cgdb.managers import TimeStepsManager
from cgdb.resources import Site
from cgdb.resources import StationGroup
from cgdb.resources import Element
from cgdb.resources import Department
from cgdb.resources import Aggregation
from cgdb.resources import ElementContext
from cgdb.resources import TimeStep
from cgdb.managers import DepartmentsManager
from cgdb.managers import ElementsCategoriesManager
from cgdb.managers import ParametersManager
from cgdb.managers import ElementContextsManager
from cgdb.managers import StationGroupsManager
from cgdb.managers import LevelsManager

from cgdb.utils import CGDBAPISession
import urllib3
urllib3.disable_warnings()

class CGDB:
    """Client class for communication with CGDB
    """
    def __init__(self, api_key, host=None):
        """ :type api_key: str
            :param api_key: api key for authenticate to api endpoint

            :type host: str or None
            :param host: api endpoint url
        """
        if host is None:
            host = "https://gari.local.czechglobe.cz:8443/api/"
        self._session = CGDBAPISession(host, api_key)
        self._sites_manager = SitesManager(client=self)
        self._data_sets_manager = DataSetsManager(client=self)
        self._departments_manager = DepartmentsManager(client=self)
        self._elements_manager = ElementsManager(client=self)
        self._elements_categories_manager = ElementsCategoriesManager(client=self)
        self._flag_lists_manager = FlagListManager(client=self)
        self._parameters_manager = ParametersManager(client=self)
        self._time_steps_manager = TimeStepsManager(client=self)
        self._element_contexts_manager = ElementContextsManager(client=self)
        self._station_networks_manager = StationNetworksManager(client=self)
        self._aggregations_manager = AggregationsManager()
        self._station_groups_manager = StationGroupsManager(client=self)
        self._levels_manager = LevelsManager(client=self)


    def sites(self):
        """Get all sites/stations
        """
        return self._sites_manager.sites()

    def site(self, code) -> Site:
        """Get site/station by mark

            :type code: str
            :param code: code of site/station
        """
        return self._sites_manager.site(code)

    def station_groups(self):
        """Get all sites groups
        """
        return self._station_groups_manager.station_groups()

    def station_group(self, code) -> StationGroup:
        """Get station group by code

            :type code: str
            :param code: code of site/station
        """
        return self._station_groups_manager.station_group(code)

    def departments(self):
        """Get all depatments groups
        """
        return self._departments_manager.departments()

    def department(self, id: str):
        """Get depatment by code

            :type code: str
            :param code: code of site/station
        """
        return self._departments_manager.department(id)

    def levels(self):
        return self._levels_manager.levels()
    
    def level(self, code):
        return self._levels_manager.level_by_code(code)

    def elements(self):
        """Get all elements groups
        """
        return self._elements_manager.elements()

    def element(self, mark: str) -> Element:
        """Get element by code

            :type code: str
            :param code: code of site/station
        """
        return self._elements_manager.element(mark)

    def element_contexts(self) -> ElementContext:
        """Get all elements contexts
        """
        return self._element_contexts_manager.element_contexts()

    def element_context_by_element(self, element_mark: str, aggregation: str, timestamp_code: str, level_code:str = None):
        return self._element_contexts_manager.element_context_by_element(element_mark, aggregation, timestamp_code, level_code)
    def create_element_context(self, element, aggregation, timeStep, level=None):
        return self._element_contexts_manager.create_element_context(element, aggregation, timeStep, level)

    def time_steps(self):
        """Get all time steps
        """
        return self._time_steps_manager.time_steps()

    def time_step(self, code) -> TimeStep:
        """Get time step by code

            :type code: str
            :param code: code of site/station
        """
        return self._time_steps_manager.time_step_by_code(code)

    def aggregations(self) -> List[Aggregation]:
        """Get aggregation by code

            :type code: str
            :param code: code of site/station
        """
        return self._aggregations_manager.aggregations()

    def create_data_set(self, site, element_code, aggregation_code, time_step_code, data_set_type, elementMarkOverride= None, level_code=None):
        """Create data set

             :type site: Site
             :param site: object of site

             :type element_code: str
             :param element_code: code of element

             :type aggregation_code: str
             :param aggregation_code: code of aggregation

             :type time_step_code: str
             :param time_step_code: code of time_step
         """
        if self.element_context_by_element(
                element_code,
                aggregation_code,
                time_step_code,
                level_code) is None:
            if level_code is not None:
                self.create_element_context(self.element(element_code), aggregation_code, self.time_step(time_step_code),
                                                   self.level(level_code))
            else:
                self.create_element_context(self.element(element_code), aggregation_code,
                                            self.time_step(time_step_code))
        self._data_sets_manager.create_data_set(site, element_code, aggregation_code, time_step_code, level_code, data_set_type, elementMarkOverride,)

    def create_station(self, name: str, code: str, gpsLatitude: float, gpsLongitude: float, altitude: int, stationNetworkCode, remark=""):
        """Create station

             :type name: str
             :param name: name of station

             :type code: str
             :param code: code of station

             :type gpsLatitude: float
             :param gpsLatitude: gps latitude

             :type gpsLongitude: float
             :param gpsLongitude: gps longitude

             :type altitude: int
            :param altitude: altitude

            :type stationNetworkCode: str
            :param stationNetworkCode: code of station network that station is member of
         """
        self._sites_manager.create_station(name, code, gpsLatitude, gpsLongitude, altitude, stationNetworkCode, remark)

    def create_station_group(self, code, name, description):
        """Create station

         :type name: str
         :param name: name of station group

         :type code: str
         :param code: code of station group

         :type description: str
         :param description: description of station group
        """
        self._station_groups_manager.create_station_group(code, name, description)

    


from cgdb.resources.aggregation import Aggregation
from cgdb.resources.dataSet import DataSets

from cgdb.resources.department import Department
from cgdb.resources.element import Element
from cgdb.resources.elementContext import ElementContext
from cgdb.resources.elementsCategory import ElementsCategory
from cgdb.resources.flag import Flag
from cgdb.resources.flagsList import FlagList
from cgdb.resources.parameter import Parameter
from cgdb.resources.site import Site
from cgdb.resources.stationNetwork import StationNetwork
from cgdb.resources.timeStep import TimeStep
from cgdb.resources.stationGroup import StationGroup
from cgdb.resources.level import Level
from cgdb.resources.overwriteStrategy import OverwriteStrategy



import json

from cgdb.resources.site import Site
from cgdb.utils.ManagerMix import ManagerMix


class SitesManager(ManagerMix):
    def __init__(self, client):
        super(SitesManager, self).__init__(client)

    def sites(self, url="sites"):
        content = self.get(url)
        sites = []

        for station_raw in content:
            sites.append(Site(**station_raw, client=self._client))

        return sites

    def site(self, mark):
        content = self.get("sites/" + mark)

        return Site(**content, client=self._client)

    def create_station(self, name, code, gpsLatitude, gpsLongitude, altitude, stationNetworkCode, remark=""):
        station_network = self._client._station_networks_manager.station_network_by_code(stationNetworkCode)
        out = {"name": name,
                "remark": remark,
                "code": code,
                "gpsLatitude": gpsLatitude,
                "gpsLongitude": gpsLongitude,
                "altitude": altitude,
                "stationNetworkId": station_network.id}
        result = self.post("sites", json.dumps(out))
        if result.status_code != 200:
            raise Exception("upload fail on server side\n Server error:"+result.content.decode("ASCII"))

    def delete_site(self, site: Site):
        self.delete(site.url)


